/*
--          This file is part of the New World OS and Objectify projects
--         Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  QRW Software
--               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   For the latest information, source code (SVN), releases, and bug tracking
--   go to:
--      http://savannah.nongnu.org/projects/objectify
--
--   For releases from Alpha_30 and up, bug and feature request tracking go to:
--      http://sourceforge.net/projects/objectify
--
--   For older bug tracking, releases and source code (CVS) prior to the
--   Alpha_30 release go to:
--      http://sourceforge.net/projects/nwos
--
--   Other related websites:
--      http://www.qrwsoftware.com
--      http://www.worldwide-database.org
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--   $Author: jsedwards $
--   $Date: 2009-10-06 08:06:23 -0600 (Tue, 06 Oct 2009) $
--   $Revision: 4384 $
--
--   NOTE: Subversion does not support the Log keyword so I have removed the
--   logs that were here when I was using CVS.  Use the "svn log" command to
--   see the revision history of this file and the objectify.h file which
--   this file was derived from.
--   (See http://subversion.tigris.org/faq.html#log-in-source)
--
*/


#ifndef REFERENCE_H
#define REFERENCE_H

#include "types.h"


#define RESERVED_PUBLIC_BLOCKS    0x10000000    /* IDs 0x00000000 to 0x0fffffff reserved */
#define RESERVED_PRIVATE_BLOCKS   0xeff00000    /* IDs 0x10000000 to 0xfff00000 reserved */
#define RESERVED_TEMPORARY_BLOCKS 0x00100000    /* IDs 0xfff00000 to 0xffffffff reserved */


/* some blocks don't come out even when dividing usable chunks per block, so don't allocate them */

#define RESERVED_PRIVATE_CHUNKS (RESERVED_PRIVATE_BLOCKS / USABLE_BLOCKS_PER_CHUNK)
/*                              (0xeff00000              / 65504)    = 61454 */

#define MAXIMUM_PUBLIC_REFERENCE   (RESERVED_PUBLIC_BLOCKS - 1)

#define MINIMUM_PRIVATE_REFERENCE  RESERVED_PUBLIC_BLOCKS

/* USE THIS in Alpha_30: */
#define MAXIMUM_PRIVATE_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS - 1)
/*                                             0x10000000 + 0xeff00000 - 1   = 0xffefffff */

/* THIS IS ONLY FOR Alpha_29 */
//#define MAXIMUM_PRIVATE_REFERENCE (RESERVED_PUBLIC_BLOCKS + (RESERVED_PRIVATE_CHUNKS * USABLE_BLOCKS_PER_CHUNK) - 1) /* 0xffeffe40 */

#define MINIMUM_TEMPORARY_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS)
#define MAXIMUM_TEMPORARY_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS + (RESERVED_TEMPORARY_BLOCKS - 1))

#define BASE_32_BIT_PRIVATE 0xfffffffe00000000ULL
#define MINIMUM_32_BIT_PRIVATE_REFERENCE (BASE_32_BIT_PRIVATE + MINIMUM_PRIVATE_REFERENCE)
#define MAXIMUM_32_BIT_PRIVATE_REFERENCE (BASE_32_BIT_PRIVATE + MAXIMUM_PRIVATE_REFERENCE)



typedef enum { Unknown_Reference, Public_Reference, Private_Reference, Temporary_Reference } Reference_Type;

typedef union { uint8 id[4]; uint32 word; } ObjRef;

static inline bool is_void_reference(ObjRef* ref)
{
    int i;
    for (i = 0; i < sizeof(ObjRef); i++) if (ref->id[i] != 0) return false;
    return true;
}


static inline bool is_same_object(ObjRef* ref1, ObjRef* ref2)
{
    int i;
    for (i = 0; i < sizeof(ObjRef); i++) if (ref1->id[i] != ref2->id[i]) return false;
    return true;
}


static inline void copy_reference(ObjRef* dst, ObjRef* src)
{
    int i;
    for (i = 0; i < sizeof(ObjRef); i++) dst->id[i] = src->id[i];
}


static inline void void_reference(ObjRef* ref)
{
    int i;
    for (i = 0; i < sizeof(ObjRef); i++) ref->id[i] = 0;
}


/* NOTE: these only work when references are 4 bytes, need to fix when size is changed */
static inline uint32 nwos_ref_to_word(ObjRef* ref) 
{
  return ((uint32)ref->id[0] << 24) | ((uint32)ref->id[1] << 16) | ((uint32)ref->id[2] << 8) | (uint32)ref->id[3];
}

static inline void nwos_word_to_ref(uint32 word, ObjRef* ref) 
{
    ref->id[0] = word >> 24;
    ref->id[1] = word >> 16;
    ref->id[2] = word >> 8;
    ref->id[3] = word;
}

extern bool nwos_is_public_reference(ObjRef* ref);
extern bool nwos_is_private_reference(ObjRef* ref);
extern bool nwos_is_temporary_reference(ObjRef* ref);
extern Reference_Type nwos_reference_type(ObjRef* ref);

#endif

