/*
--          This file is part of the New World OS and Objectify projects
--         Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010  QRW Software
--               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   For the latest information, source code (SVN), releases, and bug tracking
--   go to:
--      http://savannah.nongnu.org/projects/objectify
--
--   For releases from Alpha_30 and up, bug and feature request tracking go to:
--      http://sourceforge.net/projects/objectify
--
--   For older bug tracking, releases and source code (CVS) prior to the
--   Alpha_30 release go to:
--      http://sourceforge.net/projects/nwos
--
--   Other related websites:
--      http://www.qrwsoftware.com
--      http://www.worldwide-database.org
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--   $Author: jsedwards $
--   $Date: 2010-06-30 07:54:24 -0600 (Wed, 30 Jun 2010) $
--   $Revision: 4661 $
--
--   NOTE: Subversion does not support the Log keyword so I have removed the
--   logs that were here when I was using CVS.  Use the "svn log" command to
--   see the revisions of this file.  Since this file was originally named
--   objectify_private.h, I left the part of the log from when it was called
--   objectify_private.h below to make it easier to access.
--   (See http://subversion.tigris.org/faq.html#log-in-source)
--
-- Revision 1.87  2009/03/06 04:38:17  jsedwards
-- Added external declarations for nwos_write_bit_map, nwos_read_bit_map,
-- nwos_find_low_density_in_chunk, nwos_find_random_block_in_chunk,
-- nwos_terminate_bit_map, and nwos_find_next_empty_block.
--
-- Revision 1.86  2009/03/04 15:15:43  jsedwards
-- Added external declarations for nwos_block_offset_to_chunks and
-- nwos_ref_to_offset.
--
-- Revision 1.85  2009/03/04 15:04:47  jsedwards
-- Added external declaration for nwos_uint32_ref_to_info_index.
--
-- Revision 1.84  2009/03/02 13:55:05  jsedwards
-- Added external declarations for nwos_archive_is_block_device and
-- nwos_archive_is_file functions.
--
-- Revision 1.83  2009/03/01 15:10:47  jsedwards
-- Added external declarations for variables and functions moved into new
-- chunk_info.c file.
--
-- Revision 1.82  2009/03/01 15:05:18  jsedwards
-- Added external declaration for new nwos_realloc function.
--
-- Revision 1.81  2009/02/25 14:48:20  jsedwards
-- Removed skip_backward parameter from nwos_set_allocation_parameters external
-- declaration.
--
-- Revision 1.80  2009/01/31 15:39:25  jsedwards
-- Added NUM_FILE_CLASS_DEFINIIONS define.
--
-- Revision 1.79  2009/01/31 12:34:16  jsedwards
-- Added external declarations for nwos_ready and nwos_get_file_XXX_reference
-- functions.
--
-- Revision 1.78  2008/10/31 12:35:31  jsedwards
-- Added MAXIMUM_PUBLIC_REFERENCE, MINIMUM_PRIVATE_REFERENCE,
-- MINIMUM_TEMPORARY_REFERENCE, MAXIMUM_TEMPORARY_REFERENCE, and changed
-- MAXIMUM_VALID_PRIVATE_REFERENCE to MAXIMUM_PRIVATE_REFERENCE.
--
-- Revision 1.77  2008/10/28 10:14:58  jsedwards
-- Changed skip_forward and skip_backward parameters to unsigned.
--
-- Revision 1.76  2008/10/20 13:41:19  jsedwards
-- Changed max root transformations back to 128, now that it is an add instead
-- of multiplying by prime numbers.
--
-- Revision 1.75  2008/10/03 11:52:02  jsedwards
-- Changed MAX_TRANSFORMATIONS to MAX_ROOT_TRANSFORMATIONS and reduced it from
-- 128 to 16 because using prime numbers now.
--
-- Revision 1.74  2008/10/03 04:15:30  jsedwards
-- Added MAX_TRANSFORMATIONS and external declaration for nwos_transform_root.
--
-- Revision 1.73  2008/09/19 13:09:52  jsedwards
-- Remove #if that was testing for version string = 0029 because the compiler
-- was not happy about it.
--
-- Revision 1.72  2008/09/18 14:14:04  jsedwards
-- Changed to reserve 256 megabytes for temporary space (one mega blocks).  This
-- is kludged for Alpha_29 because the chunks don't come out even, instead of
-- the maximum valid reference being 0xffefffff it is 0xffeffe40.
--
-- Revision 1.71  2008/09/01 18:16:29  jsedwards
-- Changed nwos_create_root to not take any parameters.
--
-- Revision 1.70  2008/08/31 18:11:49  jsedwards
-- Remove duplicate declaration of nwos_read_object_from_disk which is
-- declared in objectify.h.
--
-- Revision 1.69.2.3  2008/08/13 03:46:38  jsedwards
-- Changed USABLE_BLOCKS_PER_CHUNK to the full 65536 and BLOCKS_IN_CHUNK to
-- USABLE_BLOCKS_PER_CHUNK plus BIT_MAP_BLOCKS (65568).  Also changed disk
-- header to have total_chunks instead of total_blocks, it is easier to deal
-- with.
--
-- Revision 1.69.2.2  2008/08/12 11:17:42  jsedwards
-- Removed BIT_MAP_BLOCK_MASK because it was not used anywhere and it is not
-- compatible with the new 16 megabyte chunks.
--
-- Revision 1.69.2.1  2008/08/11 13:17:44  jsedwards
-- Changed "uint16 used" member of Chunk_Info to "uint32 flags_used" and added
-- "sha1_digest" member to store chunk checksum.  Also added CHUNK_INFO_USED_MASK
-- and CHUNK_INFO_FULL_MASK defines to mask off used count in "flags_used".
--
-- Revision 1.69  2008/07/31 14:01:56  jsedwards
-- Added external declaration for nwos_read_object_from_disk_and_old_decrypt.
--
-- Revision 1.68  2008/07/19 13:33:16  jsedwards
-- Added external declarations for new disk_io_is_read_only and
-- disk_io_is_read_write functions in disk_io.c file.
--
-- Revision 1.67  2008/06/26 14:50:26  jsedwards
-- Changed nwos_initialize_backup to return bool instead of void.
--
-- Revision 1.66  2008/05/22 11:32:26  jsedwards
-- Added external declaration for nwos_disk_header.
--
-- Revision 1.65  2008/05/06 13:28:51  jsedwards
-- Changed return type of nwos_read_object_from_disk_and_decrypt to bool
-- instead of void.
--
-- Revision 1.64  2008/03/14 12:16:26  jsedwards
-- Added external reference for nwos_update_file_002_object_to_current.
--
-- Revision 1.63  2008/02/05 04:21:06  jsedwards
-- Added external declaration for nwos_get_log_file_path.
--
-- Revision 1.62  2008/02/04 03:04:32  jsedwards
-- Added external declaration for nwos_get_backup_directory_path and added
-- 'const' to nwos_initialize_backup path parameter.
--
-- Revision 1.61  2008/02/02 18:31:51  jsedwards
-- Added external declaration for nwos_get_private_objects_path.  Added
-- 'const' to nwos_public_path and nwos_private_path.  Changed StorageType to
-- AccessType in nwos_initialize_disk_io parameters.
--
-- Revision 1.60  2008/01/22 14:52:59  jsedwards
-- Added external declaration for nwos_get_public_objects_path.
--
-- Revision 1.59  2007/11/09 15:44:20  jsedwards
-- Added externs for backup.c module.
--
-- Revision 1.58  2007/10/25 00:48:38  jsedwards
-- Removed 'last_ref' paramter from nwos_generate_new_closely_spaced_id and
-- nwos_generate_new_1_in_N_id.
--
-- Revision 1.57  2007/10/19 01:48:35  jsedwards
-- Added external_declaration for the nwos_set_allocation_parameters function.
--
-- Revision 1.56  2007/09/15 18:41:53  jsedwards
-- Added ivec variable to Ref_List_First_Block and Ref_List_Extra_Block
-- structures.
--
-- Revision 1.55  2007/07/31 18:25:28  jsedwards
-- Changed 'reserved' element of Chunk_Info structure to 'index'.
--
-- Revision 1.54  2007/07/28 01:07:02  jsedwards
-- Left the "nwos_" prefix off update_file_001_object_to_current, added it.
--
-- Revision 1.53  2007/07/28 01:03:49  jsedwards
-- Added external definition for update_file_001_object_to_current.
--
-- Revision 1.52  2007/07/07 20:28:40  jsedwards
-- Move declaration of nwos_log to objectify.h.
--
-- Revision 1.51  2007/07/03 00:41:43  jsedwards
-- Added Chunk_Info structure for chunk info table (previously chunk_index).
--
-- Revision 1.50  2007/07/01 19:44:12  jsedwards
-- Upgrade to GPLv3.
--
-- Revision 1.49  2007/06/26 14:32:47  jsedwards
-- Removed private_generate_new_completely_random_id, now only used in disk_io.c.
--
-- Revision 1.48  2007/06/26 13:03:07  jsedwards
-- Changed RESERVED_PRIVATE_BLOCKS into three definitions:
--  RESERVED_PRIVATE_CHUNKS = number of chunks between 0x10000000 and 0xffffffff
--  RESERVED_PRIVATE_BLOCKS = RESERVED_PRIVATE_CHUNKS * USABLE_BLOCKS_PER_CHUNK
--  MAXIMUM_VALID_PRIVATE_REFERENCE = RESERVED_PUBLIC_BLOCKS +
--                                                  RESERVED_PRIVATE_BLOCKS - 1
--
-- Revision 1.47  2007/06/26 12:31:11  jsedwards
-- Added RESERVED_PRIVATE_BLOCKS define.
--
-- Revision 1.46  2007/06/26 12:00:22  jsedwards
-- Added "used_chunks" element to Disk_Header and external declarations
-- for "nwos_total_private_chunks", "nwos_used_private_chunks", and
-- "nwos_allocate_new_chunk".
--
-- Revision 1.45  2007/06/25 05:37:36  jsedwards
-- Added external references needed when split disk I/O functions out of
-- objectify.c and created new disk_io.c file.
--
-- Revision 1.44  2007/06/20 03:45:44  jsedwards
-- Put old 0021 Disk Header structure back in for expand_sparse.
--
-- Revision 1.43  2007/06/19 18:49:46  jsedwards
-- Removed public info from Disk Header because the public objects are now
-- separate from the private objects.
--
-- Revision 1.42  2007/05/26 14:51:08  jsedwards
-- Change create_manufacturer to have a list of URLs instead of just one.
--
-- Revision 1.41  2007/05/25 13:39:36  jsedwards
-- Add external declarations for create_url and create_public_manufacturer.
--
-- Revision 1.40  2007/05/24 11:51:51  jsedwards
-- Add external declaration for create public acronym.
--
-- Revision 1.39  2007/04/05 01:46:36  jsedwards
-- Added external declaration for 'nwos_hash_ref'.
--
-- Revision 1.38  2007/03/27 11:38:41  jsedwards
-- Add external declarations for total and used, public and private block
-- counts so that 'expand_sparse' can access them.
--
-- Revision 1.37  2007/03/26 12:35:08  jsedwards
-- Change disk header again to save time disk was prepped.
--
-- Revision 1.36  2007/03/25 12:57:26  jsedwards
-- Changed disk header yet again, separate time stamp for public and private.
--
-- Revision 1.35  2007/02/27 12:22:00  jsedwards
-- Added RESERVED_PUBLIC_BLOCKS back in and made hash_uint32_ref(), write_block(),
-- set_bit_in_map(), clear_bit_in_map(), and test_bit_in_map() external.
--
-- Revision 1.34  2007/01/27 01:51:37  jsedwards
-- Added external declaration for 'verbose' boolean.
--
-- Revision 1.33  2007/01/04 17:38:20  jsedwards
-- Add external declaration for generate_new_completely_random_id.
--
-- Revision 1.32  2006/12/30 03:49:04  jsedwards
-- Moved NUM_PUBLIC_IDS and NUM_PRIVATE_IDS to objectify.c because they are no
-- longer used by the main code (only needed now when reading an old archive).
--
-- Revision 1.31  2006/12/20 12:33:06  jsedwards
-- Changed to have two class definition refs, public and private.
--
-- Revision 1.30  2006/12/15 15:00:26  jsedwards
-- Removed total_encrypted_blocks and used_encrypted_blocks from disk header,
-- because now there are only two spaces: public and private-encrypted.
--
-- Revision 1.29  2006/12/14 13:21:33  jsedwards
-- Added external declaration of clone_class_definition function.
--
-- Revision 1.28  2006/12/08 11:30:35  jsedwards
-- Add disk header structure.
--
-- Revision 1.27  2006/12/08 10:39:57  jsedwards
-- Remove old convert 0004 to 0005 function declarations and add declaration
-- for private_create_city which is needed by the big_bang.
--
-- Revision 1.26  2006/12/07 14:07:34  jsedwards
-- Removed external references to setup routines that are no longer used.
--
-- Revision 1.25  2006/11/11 15:24:02  jsedwards
-- Added external declaration for nwos_log function.
--
-- Revision 1.24  2006/11/11 14:18:06  jsedwards
-- Added external declarations for nwos_malloc and nwos_free functions.
--
-- Revision 1.23  2006/11/11 13:57:25  jsedwards
-- Add external declaration for generate_new_public_id function.
--
-- Revision 1.22  2006/11/11 13:34:57  jsedwards
-- Added external declaration of next_public_ref.
--
-- Revision 1.21  2006/11/11 12:01:05  jsedwards
-- Update e-mail address to something that works.
--
-- Revision 1.20  2006/11/10 13:32:05  jsedwards
-- Moved Ref_List_First_Block and Ref_List_Extra_Block definitions from
-- objectify.c to objectify_private.h so that import_public_csv can use
-- them also.
--
-- Revision 1.19  2006/11/10 04:11:54  jsedwards
-- Add "write_public_object_to_disk" external declaration.
--
-- Revision 1.18  2006/10/26 01:35:58  jsedwards
-- Replaced main trunk version (1.17) with latest version from alpha_05_branch.
--
-- Revision 1.14.2.15  2006/10/22 13:09:30  jsedwards
-- Removed RESERVED_PUBLIC_BLOCKS and PRIVATE_BLOCKS_ON_DISK defines, they
-- are both now variables based upon values read from disk.
--
-- Revision 1.14.2.14  2006/10/11 13:26:28  jsedwards
-- Added bit_map_block_mask definition.
--
-- Revision 1.14.2.13  2006/10/10 07:32:36  jsedwards
-- Move bit map defines to objectify_private.h instead of spread all over.
--
-- Revision 1.14.2.12  2006/10/06 04:35:48  jsedwards
-- Renamed and rearranged the definitions of public and private, blocks and
-- ids.
--
-- Revision 1.14.2.11  2006/10/05 12:47:21  jsedwards
-- Added definitions for number of private blocks (id's) and number of
-- private blocks on the disk.
--
-- Revision 1.14.2.10  2006/09/27 13:25:13  jsedwards
-- Added reference for nwos_block_used function.
--
-- Revision 1.14.2.9  2006/09/26 13:15:35  jsedwards
-- Fixed the number of public reserved blocks (256 MB instead of 128 MB).
--
-- Revision 1.14.2.8  2006/09/26 13:00:53  jsedwards
-- Moved DEFAULT_FILE definition to config.h and added RESERVED_PUBLIC_BLOCKS
-- definition.
--
-- Revision 1.14.2.7  2006/09/18 01:36:26  jsedwards
-- Add declarations and definitions so that file.c can access block read and
-- write encryption routines directly.
--
-- Revision 1.14.2.6  2006/09/16 14:23:53  jsedwards
-- Add declaration of create_reference_list_with_existing_id.
--
-- Revision 1.14.2.5  2006/09/06 13:15:25  jsedwards
-- Removed nwos_seed_sequence function and instead pass pointers to the values
-- in the call to next_sequence function.
--
-- Revision 1.14.2.4  2006/09/03 13:54:55  jsedwards
-- Moved definition of DEFAULT_FILE here from objectify.c so the big_bang can
-- use it to create the file.
--
-- Revision 1.14.2.3  2006/09/03 11:03:24  jsedwards
-- Added declaration of overwrite function.
--
-- Revision 1.14.2.2  2006/09/02 01:09:51  jsedwards
-- Add "ref" parameter to fill_in_common_header so it can put it in header.
--
-- Revision 1.14.2.1  2006/08/18 12:55:57  jsedwards
-- Added new setup_file() routine for creating file and file path classes.
--
-- Revision 1.14  2006/01/12 02:58:28  jsedwards
-- Added external declaration of "update spelling from 0004 to 0005" routine.
--
-- Revision 1.13  2006/01/09 03:26:10  jsedwards
-- Added external reference for "upgrade persons from 0004 to 0005" routine.
--
-- Revision 1.12  2006/01/06 14:14:42  jsedwards
-- Removed external declaration of "create all fifty states".  Added external
-- declarations for "setup address", "upgrade states from 0004 to 0005", and
-- "upgrade months from 0004 to 0005".
--
-- Revision 1.11  2006/01/01 21:49:01  jsedwards
-- Moved date, phone, us_state, and word class creations out of "big bang"
-- and into the respective files.
--
-- Revision 1.10  2005/12/31 17:56:23  jsedwards
-- Added declrations for "create_class_definition" and "setup_person" routines.
--
-- Revision 1.9  2005/12/28 13:04:08  jsedwards
-- Added declarations for sequence generator routines in security.c.
--
-- Revision 1.8  2005/12/27 19:47:23  jsedwards
-- Added declaration for create root object routine.
--
-- Revision 1.7  2005/12/27 18:09:27  jsedwards
-- Changed parameters to "fill_in_common_header" and "create_english_language"
-- routines.  Added external references for two global variables: "class_
-- definition_class_ref" and "reference_list_class_ref".  Also added external
-- declarations for "set_spelling_class_ref", "set_name_class_ref", and
-- "remove_object" routines.
--
-- Revision 1.6  2005/12/21 17:10:31  jsedwards
-- Add 'nwos' to 'create_all_area_codes' routine name.
--
-- Revision 1.5  2005/12/21 17:07:36  jsedwards
-- Add declaration of create_all_area_codes.
--
-- Revision 1.4  2005/12/21 04:01:12  jsedwards
-- Added declarations for creating all states and all months.
--
-- Revision 1.3  2005/12/11 16:48:13  jsedwards
-- Added "create_english_language" declaration.
--
-- Revision 1.2  2005/12/10 15:03:36  jsedwards
-- Fixed header to say the GPL is in the LICENSE file instead of COPYING.
--
-- Revision 1.1  2005/11/26 15:19:24  jsedwards
-- External references that are private to the system (shouldn't be used by
-- 'user' programs).
--
*/

#ifndef DISK_IO_H
#define DISK_IO_H

#include "time_stamp.h"
#include "types.h"


typedef enum { Public_Archive = 0x50756241, Private_Archive = 0x50726941 } Archive_Type;


extern int  nwos_verbosity_level;


extern bool nwos_read_block_from_offset(Archive_Type archive, int64 offset, uint8 block[FILE_BLOCK_SIZE]);
extern bool nwos_write_block_to_offset(Archive_Type archive, int64 offset, uint8 block[FILE_BLOCK_SIZE]);

extern const char* nwos_public_path;
extern const char* nwos_private_path;

extern void  nwos_initialize_disk_io(AccessType type, const char* path);
extern bool  nwos_disk_io_is_read_only();
extern bool  nwos_disk_io_is_read_write();
extern bool  nwos_disk_io_is_public_only();
extern bool  nwos_disk_io_is_compressed_file(void);
extern bool  nwos_archive_is_block_device(void);
extern bool  nwos_archive_is_file(void);
extern void  nwos_write_bit_map(int chunk_index, void* map);
extern void  nwos_read_bit_map(int chunk_index, void* map);
extern void  nwos_read_chunk_info(void* chunk_info, size_t chunk_info_size_in_bytes);
extern void  nwos_write_chunk_info(void* chunk_info, size_t chunk_info_size_in_bytes);
extern void  nwos_write_empty_chunk();
extern bool  nwos_terminate_disk_io();          /* returns true if disk was modified */
extern void  nwos_change_total_chunks();


/* These are only here temorarily until the index junk can be eliminated */

#include "reference.h"

extern ObjRef* nwos_storage_index;
extern int     nwos_storage_count;

#endif

